@@ -1,6 +1,6 @@
 // 2007-04-27  Paolo Carlini  <pcarlini@suse.de>
 
-// Copyright (C) 2007, 2008, 2009, 2010, 2011 Free Software Foundation
+// Copyright (C) 2007-2019 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -18,11 +18,13 @@
 // <http://www.gnu.org/licenses/>.
 
 // { dg-do compile }
-// { dg-error "no matching" "" { target *-*-* } 1137 }
+// { dg-prune-output "no matching function .*_M_fill_initialize" }
 
 #include <vector>
 
 void f()
 {
-  std::vector<std::vector<int> > v(10, 1);
+  std::vector<std::vector<int> > v(10, 1); // { dg-error "here|no match" }
 }
+
+// { dg-prune-output "iterator_traits" }
