@@ -1,6 +1,6 @@
 // 2003-02-26 Benjamin Kosnik <bkoz@redhat.com>
 
-// Copyright (C) 2003-2018 Free Software Foundation, Inc.
+// Copyright (C) 2003-2019 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -26,8 +26,9 @@ int main()
 {
   using namespace __gnu_test;
 
-  // cplus-dem FAIL  
-  verify_demangle("_Z1jM1AFivEPS1_", "j(int (A::*)(), int (A::**)())");
+ // cplus-dem FAIL
+  verify_demangle("_Z3fooA30_A_i", "foo(int [30][])");
+
 
   return 0;
 }
