@@ -1,6 +1,6 @@
 // 2003-05-01 Petur Runolfsson <peturr02@ru.is>
 
-// Copyright (C) 2003-2017 Free Software Foundation, Inc.
+// Copyright (C) 2003-2019 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -21,7 +21,7 @@
 #include <testsuite_hooks.h>
 
 // libstdc++/12048
-void test05()
+void test04()
 {
   const char* name = "cin_unget-1.txt";
 
@@ -31,7 +31,7 @@ void test05()
   VERIFY( sbuf.sgetn(buf, 2) == 2 );
   std::wint_t c2 = sbuf.sungetc();
   VERIFY( c2 != WEOF );
-  std::wint_t c3 = std::fgetwc(file);
+  std::wint_t c3 = sbuf.sbumpc();
   VERIFY( c3 == std::char_traits<wchar_t>::to_int_type(buf[1]) );
 
   std::fclose(file);
@@ -39,6 +39,6 @@ void test05()
 
 int main ()
 {
-  test05();
+  test04();
   return 0;
 }
